# Generated by roxygen2: do not edit by hand

S3method("&",Condition)
S3method("|",Condition)
S3method(plot,milestone_time_summary)
S3method(plot,three_state_model)
S3method(print,dynamic_rng_function)
export(Arms)
export(Controllers)
export(CorrelatedPfsAndOs2)
export(CorrelatedPfsAndOs3)
export(CorrelatedPfsAndOs4)
export(DynamicRNGFunction)
export(Endpoints)
export(GraphicalTesting)
export(GroupSequentialTest)
export(Listeners)
export(Milestones)
export(PiecewiseConstantExponentialRNG)
export(Regimens)
export(StaggeredRecruiter)
export(Surv)
export(Trials)
export(add_arms)
export(arm)
export(calendarTime)
export(controller)
export(crossover)
export(doNothing)
export(endpoint)
export(enrollment)
export(eventNumber)
export(expandRegimen)
export(fitCoxph)
export(fitFarringtonManning)
export(fitLinear)
export(fitLogistic)
export(fitLogrank)
export(listener)
export(milestone)
export(qPiecewiseExponential)
export(rconst)
export(regimen)
export(remove_arms)
export(resize)
export(set_duration)
export(solveMixtureExponentialDistribution)
export(solvePiecewiseConstantExponentialDistribution)
export(solveThreeStateModel)
export(strata)
export(summarizeDataFrame)
export(summarizeMilestoneTime)
export(trial)
export(update_generator)
export(update_sample_ratio)
export(weibullDropout)
import(ggplot2)
import(survival)
importFrom(R6,R6Class)
importFrom(Rcpp,sourceCpp)
importFrom(base64enc,base64encode)
importFrom(dplyr,"%>%")
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,contains)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(emmeans,contrast)
importFrom(emmeans,emmeans)
importFrom(emmeans,regrid)
importFrom(gMCPLite,hGraph)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(htmltools,HTML)
importFrom(mvtnorm,pmvnorm)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,enquos)
importFrom(rlang,expr_text)
importFrom(rlang,sym)
importFrom(rpact,getDesignGroupSequential)
importFrom(rstudioapi,isAvailable)
importFrom(rstudioapi,viewer)
importFrom(stats,IQR)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.response)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,rweibull)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(survival,Surv)
importFrom(survival,strata)
importFrom(utils,head)
importFrom(utils,stack)
importFrom(utils,tail)
useDynLib(TrialSimulator, .registration = TRUE)
