#' @import survival ggplot2
#'
#' @importFrom dplyr %>% mutate rename bind_rows arrange row_number case_when
#' @importFrom dplyr filter select group_by summarise contains all_of matches
#' @importFrom rlang expr_text sym .data enquos :=
#' @importFrom mvtnorm pmvnorm
#' @importFrom stats as.formula pnorm model.frame model.response rexp cor runif
#' @importFrom stats IQR complete.cases median sd setNames glm pt terms uniroot
#' @importFrom stats rweibull
#' @importFrom R6 R6Class
#' @importFrom utils head tail stack
#' @importFrom emmeans emmeans contrast regrid
#' @importFrom gMCPLite hGraph
#' @importFrom rpact getDesignGroupSequential
#' @importFrom base64enc base64encode
#' @importFrom htmltools HTML
#' @importFrom grDevices dev.off png
#' @importFrom graphics axis barplot hist par
#' @importFrom rstudioapi isAvailable viewer
NULL


