

createSticker <- function(){

#   font_add_google('Caveat')
#
#   sticker(
#     subplot = image_read('~/Downloads/iStock-466742904.png'),
#     package = 'TrialSimulator',
#     s_width = 2.5,
#     s_height = 2.5,
#     s_x = 1,
#     s_y = 1.1,
#     p_y = .4,
#     p_size = 16,
#     #p_color = '#00AFFF',
#     p_family = 'Caveat',
#     white_around_sticker = TRUE,
#     h_size = 0,
#     h_color = "#FFFFFF",
#     filename = 'man/figures/logo.png',
#     dpi = 300
#   )

  invisible(NULL)

}


