% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resize.R
\name{resize}
\alias{resize}
\title{Resizing a Trial}
\usage{
resize(trial, n_patients)
}
\arguments{
\item{trial}{a trial object returned by \code{trial()}.}

\item{n_patients}{integer. Number of maximum sample size of a trial.}
}
\description{
resize a trial with a greater sample size. This function is used to update
the maximum sample size adaptively after sample size reassessment. Note that
this function should be called within action functions. It is users'
responsibility to ensure it and \code{TrialSimulator} has no way to track this.

This is a user-friendly wrapper of the member function of trial, i.e.,
\code{Trials$resize()}, which is used in vignettes. Users who are not
familiar with the concept of classes may consider using this wrapper
directly.
}
