% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFixedDesignOutput.R
\name{getFixedDesignOutput}
\alias{getFixedDesignOutput}
\title{Get simulation output in the vignette fixedDesign.Rmd}
\usage{
getFixedDesignOutput()
}
\value{
A data frame containing simulation results of 1000 replicates.
}
\description{
Internal function that retrieves precomputed simulation results.
Not meant for use by package users.
}
