% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDoseRangingOutput.R
\name{getDoseRangingOutput}
\alias{getDoseRangingOutput}
\title{Get simulation output in the vignette doseRanging.Rmd}
\usage{
getDoseRangingOutput()
}
\value{
A data frame containing simulation results of 10 replicates.
}
\description{
Internal function that retrieves precomputed simulation results.
Not meant for use by package users.
}
