% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getAdaptiveDesignOutput.R
\name{getAdaptiveDesignOutput}
\alias{getAdaptiveDesignOutput}
\title{Get simulation output in the vignette adaptiveDesign.Rmd}
\usage{
getAdaptiveDesignOutput()
}
\value{
A data frame containing simulation results of 1000 replicates.
}
\description{
Internal function that retrieves precomputed simulation results.
Not meant for use by package users.
}
