% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizeMilestoneTime.R
\name{plot.milestone_time_summary}
\alias{plot.milestone_time_summary}
\title{Plot Triggering Time of Milestones in Simulated Trials}
\usage{
\method{plot}{milestone_time_summary}(x, ...)
}
\arguments{
\item{x}{an object returned by \code{summarizeMilestoneTime()}.}

\item{...}{currently not supported.}
}
\description{
Plot Triggering Time of Milestones in Simulated Trials
}
