% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rconst.R
\name{rconst}
\alias{rconst}
\title{Generate Constant Variable}
\usage{
rconst(n, value)
}
\arguments{
\item{n}{integer. Number of observations.}

\item{value}{scalar. Value of constant observations.}
}
\description{
A random number generator returning only a constant. This can be used to
set dropout time. Currently it is the default value of dropout time, with
\code{value = Inf}.

This function can also be used as a generator of \code{endpoint()} if a
constant endpoint is needed.
}
