% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solveThreeStateModel.R
\name{plot.three_state_model}
\alias{plot.three_state_model}
\title{Plot result of three-state illness-death model}
\usage{
\method{plot}{three_state_model}(x, ...)
}
\arguments{
\item{x}{an object returned by \code{solveThreeStateModel()}.}

\item{...}{currently not supported.}
}
\description{
Plot result of three-state illness-death model
}
